#-*- coding: utf-8 -*-

#**********************************************
# OctPackage.py 
#
# Class file for the class Package
#
# Based on R2spec by Pierre-Yves chibon
#
# Copyright (C) 2011 Orion Poplawski <orion@cora.nwra.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#***********************************************

from Packager import *
from Description import *
from Spec import *
from subprocess import Popen, PIPE

import os, sys, re, datetime, glob

class OctPackage:
    '''
    Package class.
    It contains all the information about the package that oct2spec will create the spec file about.
    '''

    def __init__(self):
        ''' Initiate the object of type Package and set the attributes '''
        self.name = None
        self.source0 = None
        self.URL = None
        self.version = None
        self.date = None
        self.packager = None
        self.description = None
        self.source = None
        self.path = None
        self.created = None
        self.arch = None
        self.topdir = None
        self.sourcedir = None
        self.specdir = None
        self.tardir = None
    
    def setName(self):
        ''' Get the name of the package from the source file '''
        name = self.source
        name = name.split('-')[0]
        try:
            self.name=name.rsplit('/',1)[1]
        except IndexError, err:
            self.name=name
        
        print "* Name of the package =", self.name

    def setDate(self):
        ''' Get the current date with the correct format for the spec file '''
        date= datetime.datetime.now().strftime("%a %b %d %Y")
        self.date = date
        print "* Date of the spec file = %s" %self.date
   
    def setSource(self, source0, source):
        ''' From the url retrieve the name of the source '''
        if source != None and source0 != None:
            if source != source0.rsplit('/',1)[1] :
                sys.exit("Do not give 2 contradictory sources of information")
        elif source != None:
            try:
                self.source = source.rsplit('/',1)[1]
                self.path = source.rsplit('/',1)[0]
            except IndexError, err:
                self.source = source
                self.path = '../'
        elif source0 != None:
            self.source = source0.rsplit('/',1)[1]
            self.path = '../'
            self.source0 = source0
        else:
            sys.exit("The source are not correct ")
    
    def download(self):
        ''' 
            If the package has an url download the source file 
            Read from rpm what is the sourcedir, if the directory exists
            it downloads the file in it, else it downloads the file
            in the current working directory.
        '''
        print "\n* Retrieve the source"
        oridir = os.getcwd()
        sourcedir = self.getRPMTopDirectory('_sourcedir')
        if os.path.exists(sourcedir):
            os.chdir(sourcedir)
        
        if self.source0 != None:
            source=self.source0.rsplit('/',1)[1]
            # Check the presence of the source, download only if they are not already in the working directory
            exist = os.path.isfile(source)
            if exist == True:
                print "The file %s already exist, there is no need to download it again." %(self.source)
            else:
                cmd = "wget "+self.source0
                print cmd
                os.system(cmd)
        
        os.chdir(oridir)

    def openSource(self):
        '''
        Create a tmp directory where the tarball is opened and open
        the tarball.
        '''
        # Creates the tmp directory in which the sources are opened
        self.created = 1
        tmpdir = self.getRPMTopDirectory('_sourcedir') + '/tmp'
        try:
            os.mkdir(tmpdir)
        except Exception, err:
            self.created = 0
        #print "The folder tmp already exist, no need to create it"

        # Go to the tmp directory
        os.chdir(tmpdir)
        
        # Go into the tmp folder and untar the source
        if self.source.endswith('tar.gz'):
            cmd = "tar -zxvf " + "../" + self.source + " >/dev/null"
        elif self.source.endswith('zip'):
            cmd = "unzip ../%s >/dev/null" %self.source
        else:
            sys.exit("Your source is not a supported tarball (*.tar.gz or *.zip) %s \n" % source )
 
        #print cmd
        os.system(cmd) #or (sys.exit("The source can not be found "+source))

        #Find the name of the directory created
        self.tardir = tmpdir + '/' + glob.glob('*')[0]

        #We're assuming tmp is below us?
        os.chdir('..')

    def closeSource(self):
        '''
        Remove the tmp directory and the folder of the tarball if the 
        tmp directory has been created else just remove the folder of
        the tarball
        '''
        #print os.getcwd()
        shutil.rmtree(self.tardir)
        if self.created == 1:
            try:
                os.rmdir('tmp')
            except IOError, err:
                sys.exit('tmp directory can not be removed')
    
    def setDescription(self):
        ''' Create the object description and fill the attributes '''
        description = Description()
        description.setDescriptionFile(self)
        
        description.setVersion()
        print "* Version of the package = %s" %description.version
        description.setSummary()
        print "* Summary of the package = %s" %description.summary
        description.setDescription()
        print "* Description of the package =\n %s " %description.description
        description.setLicense()
        print "* License of the package = %s" %description.license
        description.setRequires()
        
        self.description = description
    
    def getArch(self):
        ''' Determine is the package is of the type Noarch or Arch '''
        # Save the top directory
        original_directory = os.getcwd()

        # Change directory and look for files
        os.chdir(self.tardir)
        # Look for the files and put them into a tmp file
        cmd = 'find ./ -name "*.c" -o -name "*.C" -o -name "*.cp" -o -name "*.cpp" -o -name "*.f" -o -name "*.F" -o -name "*.h" -o -name "*.H" > tmp'
        os.system(cmd)
        # Get the size of the generated file
        size = os.path.getsize('tmp')
        # If the size of this file is 0 the package should probably be noarch
        if size == 0:
            self.arch = 'noarch'
            print "* The package is = Noarch" 
        else:
            print "* The package is = Arch ~ x86 or x86_64" 
        # Remove this file
        os.system('rm tmp')
        
        # Comes back to the original directory
        os.chdir(original_directory)
        
    def setPackager(self, name, email):
        ''' Creates the object packager and fill the attributes '''
        packager = Packager()
        packager.readConfFile()
        #print name, email
        if name:
          packager.name = name
        if email:
          packager.email = email

        self.packager = packager
        print "\n","*"*50
        print "* Name of the packager =", self.packager.name
        print "* Mail of the packager =", self.packager.email    
        print "\n","*"*50

    def setUrl(self, url):
        ''' Set the given url in the URL fiel of the spec file '''
        self.URL = url

    def setOctForgeURL(self):
        ''' Set the url of a standard OctForge.net package if the option has been used '''
        self.source0 = "http://downloads.sourceforge.net/octave/%{octpkg}-%{version}.tar.gz"
        self.URL = "http://octave.sourceforge.net/%s/" %self.name
    
    def writeSpec(self, force, todo = True):
        ''' Check if the spec file already exist and generate it '''
    
        os.chdir(self.getRPMTopDirectory('_specdir'))
        # Do something if the spec file already exist
        specname = "octave-" + self.name + ".spec"
        if specname in os.listdir('./') and not force:
            text =  "The specfile octave-" + self.name + ".spec already exist !\nDo you want to overwrite it ? Y/N [N]\n"
            answer = raw_input(text)
            if answer == '':
                answer = "N"
            if answer.lower() == "n" or answer.lower() == "no":
                if specname+"-new" not in os.listdir('./'):
                    specname = specname+"-new"
                elif specname+"-new1" not in os.listdir('./'):
                    specname = specname+"-new1"
                else:
                    text = "There are already several version of the specfile in this directory, please indiquate the name you would like to give:\n"
                    specname = raw_input(text)
        
        spec = Spec(self)
        spec.finishName(self.arch)
        spec.writeSpec(specname)
        if todo: spec.writeOut()
    
    def getRPMTopDirectory(self, tag, package = None):
        ''' Reads the .rpmmacros and set the values accordingly
            Code from José Matos.
        '''
        dirname = Popen(["rpm", "-E", '%' + tag], stdout=PIPE).stdout.read()[:-1]
        # replace %{name} by the specname since it is not set

        # determine the spec name
        if self.name: specname = "octave-" + self.name
        elif package: specname = "octave-" + package
        else:         specname = "octave-unknown"

        # replace %{name} by the specname
        dirname = dirname.replace("%{name}", specname)
        dirname = dirname.replace("%name", specname)
        # replace %{version} by the specversion
        dirname = dirname.replace("%{version}", self.version)
        dirname = dirname.replace("%version", self.version)

        return dirname
        
    def moveSource(self, copyFile):
        ''' Moves the tarball to where it should be to build the RPM '''
        sourcedir = self.getRPMTopDirectory("_sourcedir")
        if os.path.exists(self.source):
            if sourcedir is not None:
                if copyFile == False:
                    text = "Do you want to copy the source (" + self.source + ") to " + sourcedir + "? Y/N [Y]\n"
                    answer = raw_input(text)
                else: answer = "y"
                if answer == '':
                    answer = "y"
                if answer.lower() == "y" or answer.lower() == "yes":
                    print "cp ./" + self.source, "to", sourcedir
                    try:
                        shutil.copy(self.source, sourcedir)
                        print "Done"
                    except Exception, err:
                        print "Can not copy the source to ",sourcedir
                        print err
                else:
                    print "Done"
    
    def findSourceOfPackage(self, name):
        ''' For a given name find in which repository the source are and returns the corresponding url.
            It uses the PACKAGES file of the repository for this.
        '''
        print "* Search source"
        import urllib, re
        dicrepo = {
                'http://downloads.sourceforge.net/octave/%s':'http://octave.sourceforge.net/packages.php',
        }
        
        for repo in dicrepo.keys():
            f = urllib.urlopen(dicrepo[repo])
            s = f.read()
            f.close()
            sourcemotif = re.compile("/octave/%s-(.*).tar.gz"%name)
            result = sourcemotif.search(s)
            if result is not None:
                print "Ok %s" %(repo)
                self.version = result.group(1).strip()
                print 'self.version' + self.version + "\n"
                pkg = "%s-%s.tar.gz" % (name,self.version)
                url = repo % pkg
                return (repo, url)
            else:
                print "Not Found: %s in %s" %(name, dicrepo[repo])
        
        return (None, None)
